# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
#
from sgmllib import SGMLParser
import socket
#import probstat
import re
import urllib2
import string

from BeautifulSoup import BeautifulSoup, Comment

options = re.DOTALL | re.UNICODE | re.MULTILINE | re.IGNORECASE
TAGS = set(('p', 'i', 'strong', 'b', 'u', 'a', 'h1', 'h2', 'h3', 'br', 'img'))
ATTRS = set(('href', 'src', 'title'))
DELETE_TAGS = ('script',)
DIV_BLOGS = set(('article-body', 'knol-content', 'post-body',
                 'post', 'entry-content', 'post-chapo', 'content',
                 'documentContent', 'message-text'))

TXT_HTML = re.compile(r'\s?\n+\s?\n?\s?', options)

EXTRACT = """\
        <div style="border: 1px solid black; padding: 8px">
 <strong>Extract from link :</strong>
 <p>
 %s
 </p>
 <br/>
</div>
"""

class Html2Txt(SGMLParser):
    def reset(self):
        SGMLParser.reset(self)
        self.pieces = []

    def handle_data(self, text):
        self.pieces.append(text)

    def handle_entityref(self, ref):
        if ref == 'amp':
            self.pieces.append("&")

    def output(self):
        return ' '.join(self.pieces)

class _Follower(object):

    _marker = '.'

    def _detect(self, entry):
        """returns a link if detected
        None otherwise
        """
        raise NotImplementedError

    def __call__(self, entry, entries):
        link = self._detect(entry)
        if link is None:
            return entry

        entry['link'] = link

        # let's get a sample of the link
        sample, encoding = self._get_sample(entry['title'], link)
        if sample is not None:
            sample = TXT_HTML.sub(r'<br/>', sample.strip())
            extract = EXTRACT % sample
            entry['summary'] = (extract.decode(encoding, 'ignore') + 
                                entry.get('summary', u'') +
                                self._marker)
        return entry
 
    def _soup(self, value):
        soup = BeautifulSoup(value)

        # removes unwanted tags (security+style)
        for comment in soup.findAll(
            text = lambda text: isinstance(text, Comment)):
            comment.extract()

        for tag in soup.findAll(True):
            # trying a few patterns
            if tag.name == 'div':
                for attr, val in tag.attrs:
                    if attr == 'class':
                        vals = val.split()
                        for wanted in DIV_BLOGS:
                            for val in vals:
                                if val == wanted:
                                    # caught a pattern
                                    c = tag.renderContents()
                                    return self._soup('<body>%s</body>' % c)
            
            if tag.name == 'p' and 'meta' in [v for n, v in tag.attrs]:
                tag.extract()
                continue
            if tag.name in DELETE_TAGS:
                tag.extract()
                continue

            if tag.name not in TAGS:
                tag.hidden = True

            tag.attrs = [(attr, val) for attr, val in tag.attrs
                        if attr in ATTRS]
        if soup.body is not None: 
            return soup.body.renderContents()
        else:
            return soup.renderContents()
    
    def _clean(self, value):
        """cleans an html page."""
        # loads and render
        parser = Html2Txt()
        parser.reset()
        parser.feed(self._soup(value))
        parser.close()
        return parser.output().strip()

    def _words(self, data):
        data = ''.join([w for w in data if w in string.ascii_letters+' '])
        return [(len(w.strip()), w.strip()) for w in data.split()]

    #def _combos(self, lists):
    #    if lists == []:
    #        return []
    #    return probstat.Cartesian(lists)
    #    if len(lists) == 1:
    #        return [[x] for x in lists[0]]
    #    
    #    res = [[i] + j for j in self._combos(lists[1:]) for i in lists[0]]
    #    print res
    #    return res

    #def _ampl_combo(self, lists):
    #    for seq in self._combos(lists):
    #        start, end = min(seq), max(seq)
    #        yield end-start, start, end     

    def _extract(self, title, content, size):
        """will try to find the best extract
        
        XXX should be better, experimenting
        """
        content_size = len(content)
        if content_size <= size:
            return content
        delta = size / 2
        
        for l, word in reversed(sorted(self._words(title))):
            positions = [m.start() for m in re.finditer(word, content)]
            if positions != []:
                pos = positions[0]
                # now findig the position of the last dot
                dots = [m.start() for m in re.finditer('\. ', content[:pos])]
                if dots == []:
                    start = pos
                else:
                    start = dots[-1] + 1
                if start + delta > content_size:
                    end = content_size
                else:
                    end = start + delta
    
                return '...' + content[start:end] + '...'

        return content[:size] + '...' 

        # finding the positions for all the words
        #def _indexes(word, content):
        #    return [e.start() for e in re.finditer(word.strip(), 
        #            content, options)]
        
         
        #positions = [_indexes(w, lcontent) for w in self._words(title) 
        #             if _indexes(w, lcontent) != []]
                
        # creating all the combos, and calculating the
        # amplitude for each
        #combos = sorted(self._ampl_combo(positions))
        #if combos == []:
        #    return content[:size]
        #ampl, start, end = combos[0]
        #if ampl > size:
        #    end = start + size
            
        #return '...' + content[start:end] + '...'
 
    def _get_sample(self, title, link, size=1000):
        """get the page, extract part of it if it is some text.
        
        tries to find the words of the title, to extract 
        the most meaningful part of the page.
        """
        charset = 'utf-8'
        try:
            page = urllib2.urlopen(link)
            if 'content-type' in page.headers.keys():
                content_type = page.headers['content-type'].split(';')
                type_ = content_type[0].strip().lower()
                if type_ not in ('text/html', 'text/plain', 'test/rst'):
                    return None, None
                if len(content_type) > 1:
                    charset = content_type[1].split('=')[-1]
            content = page.read()

        except (urllib2.HTTPError, urllib2.URLError, socket.timeout):
            return None, None
        body = self._clean(content)
        return self._extract(title.encode(charset), body, size), charset

class RedditFollower(_Follower):
    """
    Will detect a reddit-like post and fill the summary
    of the entry with an extract of the target page 
    by following the link.

    The filter tries to pick up the best extract out
    of the page.
    """
    pattern = r'submitted .*? <br /> <a href="(.*?)">\[link\]</a> <a href=".*?">\[.*?comments\]</a>$'

    def _detect(self, entry):
        summary = entry.get('summary', '')

        link = re.search(self.pattern, summary, options) 
        if link is None:
            return None

        # reddit-like detected
        return link.groups()[0]

class DeliciousFollower(_Follower):
    """
    Will detect a delicious-like post and fill the summary
    of the entry with an extract of the target page 
    by following the link.

    The filter tries to pick up the best extract out
    of the page.
    """
    def _detect(self, entry):
        title_detail = entry.get('title_detail')
        if title_detail is not None and 'delicious.com' in title_detail['base']:
            return entry['link']
        comments = entry.get('comments', '')
        if comments.startswith('http://delicious.com/url/'):
            return entry['link']
        return None




# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
#
import urllib2
from atomisator.filters.followers import RedditFollower, DeliciousFollower
from nose.tools import *

DATA = """
<html>
 <body>
    The simplest way to use this module is to call the urlopen function,
    which accepts a string containing a URL or a Request object (described
    below).  It opens the URL and returns the results as file-like
    object; the returned object has some extra methods described below.
    
    The OpenerDirector manages a collection of Handler objects that do
    all the actual work.  Each Handler implements a particular protocol or
    option.  The OpenerDirector is a composite object that invokes the
    Handlers needed to open the requested URL.  For example, the
    HTTPHandler performs HTTP GET and POST requests and deals with
    non-error returns.  The HTTPRedirectHandler automatically deals with
    HTTP 301, 302, 303 and 307 redirect errors, and the HTTPDigestAuthHandler
    deals with digest authentication.
    
    urlopen(url, data=None) -- basic usage is that same as original
    urllib.  pass the url and optionally data to post to an HTTP URL, and
    get a file-like object back.  One difference is that you can also pass
    a Request instance instead of URL.  Raises a URLError (subclass of
    IOError); for HTTP errors, raises an HTTPError, which can also be
    treated as a valid response.
    
    build_opener -- function that creates a new OpenerDirector instance.
    will install the default handlers.  accepts one or more Handlers as
    arguments, either instances or Handler classes that it will
    instantiate.  if one of the argument is a subclass of the default
    handler, the argument will be installed instead of the default.
    
    install_opener -- installs a new opener as the default opener.

    HTTPRedirectHandler
 </body>
</html>
"""

class Url:

    headers = {'content-type': 'text/html; charset=utf-8'}
    def __init__(self,  d):
        self._data = d

    def read(self):
        return self._data

def setupurl():
    urllib2.old = urllib2.urlopen
    def open(*args):
        return Url(DATA)
    urllib2.urlopen = open

def teardownurl():
    urllib2.urlopen = urllib2.old
    del urllib2.old

@with_setup(setupurl, teardownurl)
def test_reddit():
    red = RedditFollower()
    summary = ('submitted by <a href="http://www.reddit.com/user/'
               'sebastianavina">sebastianavina</a> <br /> <a href='
               '"http://www.reddit.com/r/python">[link]</a> <a href'
               '="http://www.reddit.com/r/Python/comments/6ytrh/pytho'
               'n_subreddit_now_skinned_as_pythonorg/">[22 comments]</a>')

    e = {'summary': summary,
         'title': ('HTTPRedirectHandler and HTTPDigestAuthHandler' 
                   ' (and more)')}
    entry = red(e, [])
    assert 'HTTPRedirectHandler' in entry['summary']



@with_setup(setupurl, teardownurl) 
def test_delicious():
    delicious = DeliciousFollower()
    
    e = {'comments': 
            'http://delicious.com/url/45bb751bb4dd1f4a291c91cf31c43511',
         'title': ('HTTPRedirectHandler and HTTPDigestAuthHandler' 
                   ' (and more)'),
         'link': 'http://link'}
    entry = delicious(e, [])
    assert 'HTTPRedirectHandler' in entry['summary']




# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
#
import os
from nose.tools import *

from atomisator.filters import StopWords
from atomisator.filters import BuzzWords
from atomisator.filters import Doublons
from atomisator.filters import ReplaceWords

def test_stop():
    stopfile = os.path.join(os.path.dirname(__file__), 
                           'words.txt')
    sw = StopWords()
    entry = {'title': 'the title', 
            'summary': 'viagra info'}
    entries = []
    entry = sw(entry, entries, stopfile)
    assert_equals(entry, None)

    entry = {'title': 'the title', 
            'summary': 'info'}
    res = sw(entry, entries, stopfile)
    assert_equals(entry, res)

    # based on regepx
    entry['summary'] = 'start with start'
    res = sw(entry, entries, stopfile)
    assert_equals(res, None)


def test_buzz():
    buzzfile = os.path.join(os.path.dirname(__file__), 
            'words.txt')

    bw = BuzzWords()
    entry = {'title': 'the title', 
            'summary': 'viagra info'}
    entries = []
    entry = bw(entry, entries, buzzfile)
    assert_equals(entry['title'], '[viagra] the title')

    entry = {'title': 'the title', 
            'summary': 'info'}
    res = bw(entry, entries, buzzfile)
    assert_equals(res, None)

def test_doublons():
    db = Doublons()
    entry = {'title': 'the title', 
            'summary': 'info'}
    class E:
        link = summary = title = url = ''

    e = E()
    e.summary = 'info'
    entries = [e]
    entry = db(entry, entries)
    assert_equals(entry, None)

    entry = {'title': 'the title', 
            'summary': 'info'}
    res = db(entry, [])
    assert_equals(res, entry)

def test_replace():
    rw = ReplaceWords()
    entry = {'title': 'the title', 'summary': 'info'}
    replfile = os.path.join(os.path.dirname(__file__), 
                            'replace.txt')
    res = rw(entry, [], replfile)
    assert_equals(res['title'], 'the tixxxtle')
    assert_equals(res['summary'], 'info sp')

    entry = {'title': 'the title', 
             'summary': 'info not interesting'}
    res = rw(entry, [], replfile)
    assert_equals(res['title'], 'the tixxxtle')
    assert_equals(res['summary'], 'info sp ')





# -*- encoding: utf-8 -*-                                                      
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>                             
# 
from setuptools import setup, find_packages
import os
from os.path import join
from distutils.core import Extension 

version = '0.1.9'

long_description = open("README.txt").read()

includes = [join('probstat', 'include'), 
             join('probstat', 'python')]
libraries = [] 

files = [
         join('probstat', 'base', 'cartesian_base.c'),
         join('probstat', 'base', 'permutation_base.c'),
         join('probstat', 'base', 'combination_base.c'),
         join('probstat', 'base', 'pqueue_base.c'),
         join('probstat', 'python', 'cartesian.c'),
         join('probstat', 'python', 'permutation.c'),
         join('probstat', 'python', 'combination.c'),
         join('probstat', 'python', 'pqueue.c'),
         join('probstat', 'python', 'stats_module.c'),
        ]


classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ]

entry_points = {"atomisator.filters": ["stopwords = atomisator.filters:StopWords",
                                       "buzzwords = atomisator.filters:BuzzWords",
                                       "doublons = atomisator.filters:Doublons",
                                       "spam = atomisator.filters:Spam",
                                       "replace = atomisator.filters:ReplaceWords",
                                       "autotag = atomisator.filters:AutoTag",
                                       "reddit = atomisator.filters.followers:RedditFollower",
                                       "delicious = atomisator.filters.followers:DeliciousFollower"]}

setup(name='atomisator.filters',
      version=version,
      description="Expert Python Programming - Filters",
      long_description=long_description,
      classifiers=classifiers,
      keywords='python best practices',
      author='Tarek Ziade',
      author_email='tarek@ziade.org',
      url='http://atomisator.ziade.org',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['atomisator'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'BeautifulSoup',
      ],
      entry_points=entry_points,
      #ext_modules = [Extension("probstat", files,
      #                         libraries = libraries,
      #                         include_dirs =  includes,
      #                        )
      #              ],

      )



# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
#
import re

_files = {}
options = re.DOTALL | re.UNICODE | re.MULTILINE | re.IGNORECASE

class FileFilter(object):

    def __init__(self):
        self._cached_expressions = {}

    def _comp(self, line):
        line = line.strip()
        return line, re.compile(line, options)

    def _get_texts(self, entry):
        return entry['title'], entry.get('summary', '')

    def _match(self, entry, path):
        if path not in self._cached_expressions:
            patterns = [l for l in open(path).read().split('\n')
                        if l.strip() != '']
            expr = re.compile('|'.join(patterns), options)
            self._cached_expressions[path] = expr
        text = '\n'.join(self._get_texts(entry))
        return self._cached_expressions[path].search(text) 

class StopWords(FileFilter):
    """
    Filter off entries that matches regular expressions.
    Expressions are given in a file, one per line.
    The file path is provided as argument.
    """ 
    def __call__(self, entry, entries, path):
        if self._match(entry, path) is not None:
            return None
        return entry

class ReplaceWords(FileFilter):
    """
    Replace one expression to another one.
    Expressions are given in a file.
    Each line is the matching expression, 
    followed by the replacement string.
    Elements are separated by the ":::" expression.

    The file path is provided as argument.
    """
    def _replace(self, entry, path):
        if path not in self._cached_expressions:
            lines = [self._comp(l) 
                     for l in open(path).read().split('\n')
                     if l.strip() != '']
            self._cached_expressions[path] = lines
        for key in ('summary', 'title'):
            for comp, repl in self._cached_expressions[path]:
                entry[key] = comp.sub(repl, entry[key]) 
        return entry

    def _comp(self, line):
        line = line.strip()
        spl = line.split(':::')
        if len(spl) == 1:
            exp, repl = spl[0], ''
        else:
            exp, repl = spl
        return re.compile(exp, options), repl

    def __call__(self, entry, entries, path):
        return self._replace(entry, path) 

class BuzzWords(FileFilter):
    """
    Keep only the entries that matches regular expressions.
    Expressions are given in a file, one per line.
    The file path is provided as argument.
    """ 

    def __call__(self, entry, entries, path):
        """Keeps entries based on keywords"""
        m = self._match(entry, path)
        if m is not None:
            entry['title'] = '[%s] %s' % (m.group(), entry['title'])
            return entry
        return None

class Doublons(object):
    """
    Entries are unique by their url, but sometimes you can have doublons.
    
    This filter filters off entries that are already in the database.
    By looking at the summaries.

    XXX todo : use levensthein distance here

    """ 
    def _clean(self, st):
        if st is None:
            return None
        return st.lower().strip()
    
    def __call__(self, entry, entries):
        link = self._clean(entry.get('link', ''))
        summary = self._clean(entry.get('summary', ''))
        for e in entries:
            if link == self._clean(e.link):
                return None
            if summary == self._clean(e.summary):
                return None
        return entry

class Spam(object):
    """
    Tries to remove spamy entries.
    """
    def __call__(self, entry, entries):
        if 'summary' in entry and entry['summary'].strip() == '':
            return None
        return entry

class AutoTag(FileFilter):
    """
    Automatically tag entries when words or regular expression
    from a file are found into this entry.
    """ 
    def __call__(self, entry, entries, path):
        """Keeps entries based on keywords"""
        match = self._match(entry, path)
        if match is not None:
            entry['tags'] = [{'term': tag} for tag in  match.group()]
        return entry

